from argparse import ArgumentParser, Namespace
from collections.abc import Sequence
from typing import Optional

from ..config import load_config
from .base import BaseCommand, Command
from .room import RoomCommand
from .sync import SyncCommand

CONFIG_DEFAULT = '/etc/fsmi-matrix-bot/config.cfg'


class RootCommand(BaseCommand):
    help = 'Bot for managing the FSMI Matrix server'

    def __init__(self) -> None:
        self.parser = ArgumentParser(description=self.help)
        self._add_arguments(self.parser)

    def _add_arguments(self, parser: ArgumentParser) -> None:
        parser.add_argument(
            '--config', '-c',
            default=CONFIG_DEFAULT,
            help='Path to configuration file',
        )

        subparsers = parser.add_subparsers(metavar='COMMAND', required=True)
        RoomCommand.add_as_subparser(self, subparsers)
        SyncCommand.add_as_subparser(self, subparsers)
        WhoAmICommand.add_as_subparser(self, subparsers)

    def parse_and_execute(self, args: Optional[Sequence[str]] = None) -> None:
        self.args = self.parser.parse_args(args)
        load_config(self.args.config)
        self.args._handle(self.args)


class WhoAmICommand(Command):
    command = 'whoami'
    help = 'Get Information about the bot user on the homeserver'

    def handle(self, args: Namespace) -> None:
        from .. import get_client
        print(get_client().whoami())
        print(get_client().token)
