import argparse
from typing import Optional


class BaseCommand:
    help: Optional[str] = None


class Command(BaseCommand):
    command: str

    def __init__(self, parent: BaseCommand) -> None:
        self.parent = parent

    @classmethod
    def add_as_subparser(
        cls,
        parent: BaseCommand,
        parser: argparse._SubParsersAction
    ) -> None:
        command = cls(parent)

        subparser = parser.add_parser(cls.command, help=cls.help)
        subparser.set_defaults(_handle=command.handle)
        command.add_arguments(subparser)

    def add_arguments(self, parser: argparse.ArgumentParser) -> None:
        pass

    def handle(self, args: argparse.Namespace) -> None:
        raise NotImplementedError(
            f'No handler implemented for command "{self.command}"'
        )
